/* E. Farhat
   E. Herrera
   R. Jordan
   A. Wilkes
*/

/* main.c for our Final Project - TAMF */


#include "xbasic_types.h"
#include "xio.h"
#include "xintc_l.h"
#include "xuartlite_l.h"

/* // defined in isr.c */
extern void uart_handler(void *callback); 
extern int uart_interrupt_count;
extern char uart_character [256];

#define W 480
#define H 360
#define VGA_START 0x00800000
#define RED 0xF800
#define GREEN 0x07E0
#define BLUE 0x001F
#define NOT_VERT_SYNC (XIo_In32(0x01800014))
#define VERT_SYNC     (!XIo_In32(0x01800014))
#define NOT_HORZ_SYNC (XIo_In32(0x01800010))
#define HORZ_SYNC     (!XIo_In32(0x01800010))


/*
 * setup_interrupts: Initialize the interrupt sources and handlers
 *
 * Should be called once when the system starts
 *
 * The main _interrupt_handler() function from Xilinx
 *7
 * Saves and restores CPU context, etc.
 *
 * Sees which interrupts are pending, and for each it
 *     acknowledges the interrupt and
 *     calls a user-defined interrupt handler in Xintc_InterruptVectorTable
 *
 * Place interrupt service routines in isr.c to ensure they are placed in
 * the proper memory segment.
 */

void setup_interrupts()
{
  /*
   * Reset the interrupt controller peripheral
   */

  /* Disable the interrupt signal */
  XIntc_mMasterDisable(XPAR_INTC_SINGLE_BASEADDR);

  /* Disable all interrupt sources */
  XIntc_mEnableIntr(XPAR_INTC_SINGLE_BASEADDR,0);

  /* Acknowledge all possible interrupt sources
     to make sure none are pending */
  XIntc_mAckIntr(XPAR_INTC_SINGLE_BASEADDR, 0xffffffff);

  /*
   * Install the UART interrupt handler
   */

  XIntc_InterruptVectorTable[XPAR_INTC_MYUART_INTERRUPT_INTR].Handler =
    uart_handler;

  /*
   * Enable interrupt sources
   */

  /* Enable CPU interrupts */
  microblaze_enable_interrupts();

  /* Enable interrupts from the interrupt controller */
  XIntc_mMasterEnable(XPAR_INTC_SINGLE_BASEADDR);

  /* Tell the interrupt controller to accept interrupts from the UART */
  XIntc_mEnableIntr(XPAR_INTC_SINGLE_BASEADDR, XPAR_MYUART_INTERRUPT_MASK);
  
  /* Enable UART interrupt generation */
  XUartLite_mEnableIntr(XPAR_MYUART_BASEADDR);
}

//Creating our Structure

typedef struct {
  int blank_len;
  int step_size;
  int cnt_pix;
} line_info;

line_info lines[H];
line_info lines2[H];


//This computes all the major operations required to calculate our necessary
//values.

void compute_linfo()
{
  int i;
  int npix;
 
  for(i=0;i<H;i++){

    npix = i;
 
    lines[i].blank_len = (W-npix)/2;
    lines[i].step_size = (W<<12) / (npix +1);
    lines[i].cnt_pix= npix + (W-npix)/2;
  }
  
  npix = H;
  for(i=0;i<H;i++){
    
    if(i <= 180)
      npix--;
    else
      npix=i;
 
    lines2[i].blank_len = (W-npix)/2;
    lines2[i].step_size = (W<<12) / (npix +1);
    lines2[i].cnt_pix= npix + (W-npix)/2;
    
  }
}

// This function will create a triangle shape where our heads are compressed
// and our bodies are semi-normal

void Tri_comp()
{
  char c;
  int i, x;
 
  int start_addr;
 
  int j =0;
  int nl;

 

  while(1)
    {
      while(VERT_SYNC);
      if(j == 500)
	return;
      nl=-88;
      start_addr = -480*90;
   
      while(1)
	{
	  while(NOT_HORZ_SYNC);
	  
	  // Blank length
	  XIo_Out32(0x01800000, lines[nl].blank_len);
	  
	  // Start address
	  XIo_Out32(0x01800004, start_addr);
	  
	  // Step size
	  XIo_Out32(0x01800008, lines[nl].step_size);
	  
	  // Cnt pix
	  XIo_Out32(0x0180000C, lines[nl].cnt_pix);

	  start_addr += 480;
	  nl ++;
	 	 
	  if (VERT_SYNC)
	    break;
	  
	  while(HORZ_SYNC);
	}//second while
      j++;
    }
    
}

//This function will take our image and form it into an hourglass shape

void Hour_Glass()
{
  char c;
  int i, x;
 
  int start_addr;
 
  int j =0;
  int nl;

   while(1)
    {
      while(VERT_SYNC);
      if(j == 500)
      	return;
      nl=-90;
      start_addr = -480*90;
   
      while(1)
	{
	  while(NOT_HORZ_SYNC);
	  
	  // Blank length
	  XIo_Out32(0x01800000, lines2[nl].blank_len);
	  
	  // Start address
	  XIo_Out32(0x01800004, start_addr);
	  
	  // Step size
	  XIo_Out32(0x01800008, lines2[nl].step_size);
	  
	  // Cnt pix
	  XIo_Out32(0x0180000C, lines2[nl].cnt_pix);

	  start_addr += 480;
	  nl ++;
	 	 
	  if (VERT_SYNC)
	    break;
	  
	  while(HORZ_SYNC);
	}//second while
      j++;
    }
    
}

//This will show the full image with normal dimensions

void Full()
{
  int j=0;
  int start_addr;
  int i = 1<<12;
 
  while(1)
    {
      while(VERT_SYNC);
      if(j==10)
	return;
      start_addr = -480*90;
      
      while(1)
	{
	  while(NOT_HORZ_SYNC);
	  
	  // Blank length
	  XIo_Out32(0x01800000, 3);
	  
	  // Start address
	  XIo_Out32(0x01800004, start_addr);
	  
	  // Step size
	  XIo_Out32(0x01800008,i);
	  
	  // Cnt pix
	  XIo_Out32(0x0180000C, 480 );
	  
	  start_addr += 480;
	  
	  
	  if (VERT_SYNC)
	    break;
	  
	  while(HORZ_SYNC);
	}//second while
      j++; 
    }
}    

/* This will horizontally compress our image and once it reaches
   the center of the screen it will begin to invert and once again go back 
   to its original size
*/

void Horz_comp()
{
  char c;
  int i, x;
 
  int start_addr;
 
  int j =0;
  int nl;

  x=0;
  nl=H ;
  while(1)
    {
      while(VERT_SYNC);
      
     

      /*compresses*/
      if(j  < H-1){
	nl--;	
	start_addr = -W*90;
	i=lines[nl].step_size; 
      }
      /*start expanding*/
      else if(j < 2*H  ){
	start_addr = -W*90 +W;
	
        i= - lines[nl].step_size;
	nl++;
      }	
      /*delay and return*/
      else if (j<3*H ){
	nl--;	
	start_addr = -W*90 + W  ;
	i=-lines[nl].step_size;

	//return;
      }  else{
	nl++;
	start_addr = -W*90;
	i=lines[nl].step_size; 
	if(nl==H+1)
	  return;
      }


      while(1)
	{
	  while(NOT_HORZ_SYNC);
	  
	  // Blank length
	  XIo_Out32(0x01800000, lines[nl].blank_len);
	  
	  // Start address
	  XIo_Out32(0x01800004, start_addr);
	  
	  // Step size
	  XIo_Out32(0x01800008, i);
	  
	  // Cnt pix
	  XIo_Out32(0x0180000C, lines[nl].cnt_pix);

	
	    start_addr += W;
	
	 	 
	  if (VERT_SYNC)
	    break;
	  
	  while(HORZ_SYNC);
	}//second while

      j++;
    }
        
}


int main()
{
  char *prt_char;
  char *read_char;
  int j;
  
  
  // Enable the instruction cache: makes the code run 6 times faster
   microblaze_enable_icache();
  
  print("Hello and welcome to our final project!\r\n");
  print("Here is our MENU: \r\n");
  print("~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~\r\n"); 
  print("~ To compress and invert us please enter 'H' ~\r\n"); 
  print("~ To see us in an hourglass please enter 'G' ~\r\n"); 
  print("~ To see us in a pyramid please enter 'T'    ~\r\n");
  print("~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~\r\n");
  print("Please note our program is NOT case sensitive.\r\n");
  print("What would you like to display?");
  j =0;


  compute_linfo();
  
   setup_interrupts();

  for (;;) {
    
     prt_char = &uart_character[j];
   
    
    //We will disable the interrupt for a short while    
    microblaze_disable_interrupts();
    read_char = &uart_character[uart_interrupt_count%256];
    microblaze_enable_interrupts();   
   
    if(read_char == prt_char){ // buffer is not empt
	Full(); 
    }
    
    /*handles input from minicom*/
    if(read_char != prt_char){ // buffer is not empty 
      
      /*if h is pressed will begin Horz_comp() */  
      if ( *prt_char=='t' || *prt_char=='T'){             
	Tri_comp();
	
      }//end if
      
      /*or if t  is entered, then makes a triangle form of the image */
      else if (*prt_char=='h'|| *prt_char=='H'){
	Horz_comp();

      }
      /*or if t  is entered, then makes a triangle form of the image */
      else if (*prt_char=='g'|| *prt_char=='G'){
	Hour_Glass();
      }
      
      
      /* otherwise, print to the screen */
      else{
       	Full();
	
      }
      
      j=(j+1)%256;
      
    }//if
    
    
    
  
    
  }//for  
  return 0;
}
